@@ -7,7 +7,7 @@ from argparse import ArgumentParser, Action, RawDescriptionHelpFormatter
 from glob import glob
 from multiprocessing import Pool
 
-from eis1600.helper.repo import travers_eis1600_dir, get_files_from_eis1600_dir
+from eis1600.helper.repo import travers_eis1600_dir, get_files_from_eis1600_dir, read_files_from_readme, write_to_readme
 from eis1600.markdown.methods import convert_to_eis1600
 
 
@@ -26,7 +26,7 @@ class CheckFileEndingAction(Action):
 if __name__ == '__main__':
 
     arg_parser = ArgumentParser(prog=sys.argv[0], formatter_class=RawDescriptionHelpFormatter,
-                                         description='''Script to convert mARkdown file(s) to EIS1600_tmp file(s).
+                                description='''Script to convert mARkdown file(s) to EIS1600TMP file(s).
 -----
 Give a single mARkdown file as input
 or 
@@ -56,7 +56,7 @@ Use -e <EIS1600_repo> to batch process all mARkdown files in the EIS1600 directo
         if not input_dir[-1] == '/':
             input_dir += '/'
 
-        print(f'Convert mARkdown files from {input_dir}, save resulting EIS1600_tmp files to {output_dir}')
+        print(f'Convert mARkdown files from {input_dir}, save resulting EIS1600TMP files to {output_dir}')
 
         infiles = glob(input_dir + '*.mARkdown')
         if not infiles:
@@ -73,9 +73,13 @@ Use -e <EIS1600_repo> to batch process all mARkdown files in the EIS1600 directo
             p.starmap_async(convert_to_eis1600, params).get()
     elif args.eis1600_repo:
         input_dir = args.eis1600_repo
+        if not input_dir[-1] == '/':
+            input_dir += '/'
 
-        print(f'Convert mARkdown files from the EIS1600 repo (only if there is not an EIS1600_tmp file yet)')
-        infiles = get_files_from_eis1600_dir(input_dir, ['*.mARkdown', '*.inProcress', '*.completed'], '*.EIS1600*')
+        print(f'Convert mARkdown files from the EIS1600 repo (only if there is not an EIS1600TMP file yet)')
+        files_list = read_files_from_readme(input_dir, '# Texts with fixed poetry\n')
+        infiles = get_files_from_eis1600_dir(input_dir, files_list, ['*.mARkdown', '*.inProcress', '*.completed'],
+                                             '*.EIS1600*')
         if not infiles:
             print(
                 'There are no more mARkdown files to process')
@@ -85,6 +89,8 @@ Use -e <EIS1600_repo> to batch process all mARkdown files in the EIS1600 directo
 
         with Pool() as p:
             p.starmap_async(convert_to_eis1600, params).get()
+
+        write_to_readme(input_dir, infiles, '# Texts converted into `.EIS1600TMP`\n', '.EIS1600TMP')
     else:
         print(
             'Pass in a <uri.mARkdown> file to process a single file or use the -e option for batch processing'
